
# q-expansion of newform 100.10.c.c, downloaded from the LMFDB on 11 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 100
weight = 10
poly_data = [400, 0, -39, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, -19, 0, 1], 2], [[0, 472, 0, -8], 1], [[-624, 0, 32, 0], 1]]

hecke_ring_character_values = [[51, [1, 0, 0, 0]], [77, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, -13, 0, 1], [0, 0, 0, 0], [0, 19, 0, -69], [51360, 0, 3, 0], [0, 8957, 0, 564], [0, -15801, 0, -348], [-68636, 0, -516, 0], [0, -33273, 0, -6393], [3446874, 0, 588, 0], [145916, 0, -2625, 0], [0, -563069, 0, 59976], [14886726, 0, 7218, 0], [0, -585409, 0, -99615], [0, -3124665, 0, -42573], [0, 470889, 0, -212532], [46465428, 0, 37398, 0], [97836962, 0, 72144, 0], [0, 10988371, 0, -203139], [155603508, 0, -139761, 0], [0, -4961203, 0, -2047716], [-271130888, 0, -7002, 0], [0, -62845785, 0, 192957], [231145926, 0, -542196, 0], [0, -83585837, 0, 2902956], [-593410074, 0, -351204, 0], [0, -22167493, 0, 960723], [0, 24904263, 0, -5435043], [726810718, 0, -117708, 0], [0, -76344231, 0, -13181592], [0, -442546157, 0, 7684191], [438215520, 0, -2972379, 0], [0, 456254955, 0, -16951536], [-3276221276, 0, 2110338, 0], [3893692326, 0, 1597716, 0], [4241962580, 0, -5291937, 0], [0, 180271615, 0, 631704], [0, 520032767, 0, -33654435], [0, 362283, 0, 109829019], [0, 603375273, 0, 39181368], [20120817132, 0, 3118458, 0], [1868195030, 0, 943752, 0], [4045319028, 0, 21172353, 0], [0, 1496133833, 0, 71205372], [0, -1028429637, 0, 4522884], [9530496952, 0, -9052512, 0], [7052087504, 0, -17340987, 0], [0, -1739205793, 0, -229646085], [0, 810798855, 0, 4030497], [21212864146, 0, 5898624, 0], [0, -2681724903, 0, -333620172], [-11554367976, 0, 19767822, 0], [16175404814, 0, 14015658, 0], [50380749600, 0, -19045545, 0], [0, 628171239, 0, 277620312], [0, 10007270763, 0, -4792821], [-41240171058, 0, -81733944, 0], [-15073810540, 0, 30238857, 0], [0, 1188446887, 0, 1099932648], [44344622406, 0, -91084506, 0], [0, -10167048613, 0, -712207707], [0, 9704631021, 0, 242777520], [0, 10227452371, 0, 608327541], [-45587739828, 0, -17901291, 0], [0, 13732665809, 0, 1390226376], [0, 16122714615, 0, 259723932], [-137606920792, 0, -16151463, 0], [0, 12653099059, 0, -1506593784], [0, 7412623203, 0, -1857192975], [-360018204950, 0, -88361208, 0], [0, 20389182453, 0, -1071014904], [-79341904344, 0, 181753296, 0], [0, -5437577681, 0, 576658179], [0, 37206248537, 0, -962419128], [107446254268, 0, 143813580, 0], [0, -38082269241, 0, 2341372227], [-183838550874, 0, 45222024, 0], [0, -38766734333, 0, 2227520076], [140179190370, 0, 296957520, 0], [-569175787166, 0, 243184218, 0], [-772137282756, 0, 293906010, 0], [636913820666, 0, -75676212, 0], [459921283236, 0, 567943749, 0], [0, -24244126483, 0, 4017285204], [413950061752, 0, -480514896, 0], [0, 24318398439, 0, 1246459137], [-167496460806, 0, 33558270, 0], [0, -104284302473, 0, -2155836672], [-513733206210, 0, 508798188, 0], [0, -69865852165, 0, 269391303], [0, 102667937763, 0, -1390279275], [-235494243816, 0, 621755022, 0], [0, 99256145911, 0, 5394966303], [70709287200, 0, 162733935, 0], [-658472549276, 0, 103166856, 0], [0, 359922027, 0, 11873375979], [-1106148875046, 0, 278412336, 0], [17965062714, 0, -637614720, 0], [0, 48613932635, 0, 3686895693], [1170694100654, 0, 1567256100, 0], [0, 207732082615, 0, 7212961233], [0, 44262080499, 0, 351145080], [0, 202036785675, 0, -2408422911], [-450217464030, 0, -2078771070, 0], [-3522141156952, 0, -372823095, 0], [0, 38129954575, 0, 525114864], [0, 282340323591, 0, 106720317], [0, -271820462475, 0, -9624184224], [-3930607558080, 0, -382971834, 0], [2072229737270, 0, -1797423906, 0], [0, 8890363951, 0, 34295369859], [0, -173104902115, 0, -4947253596], [0, 135232339047, 0, -22350324708], [2279799676300, 0, -808110144, 0], [-778435665628, 0, 1931941443, 0], [-2156581307538, 0, 3409013178, 0], [0, -329518621285, 0, 7139276073], [0, 385909909263, 0, 7099379751], [0, 211911641793, 0, 1314857004], [-7993719098436, 0, 417006318, 0], [-446412433462, 0, -1097796168, 0], [0, -126912886603, 0, -62205321492], [0, 699861939963, 0, 13452098124], [0, -472345453713, 0, -31960769439], [-381560883352, 0, 341077923, 0], [4976157968130, 0, -144032532, 0], [2322921561154, 0, -6970015956, 0], [4840236023064, 0, -4463710362, 0], [0, -962059142837, 0, -11798915733], [0, 598591823681, 0, -34563357504], [6468463294540, 0, 5637397878, 0], [0, 919820480775, 0, 22752569367], [9355741693436, 0, -397136313, 0], [0, -1015602587105, 0, 15591317904], [1377068364522, 0, 6960947280, 0], [-3401690820482, 0, -285310932, 0], [0, -162402388083, 0, -15951738900], [0, -267420420617, 0, 37808296713], [0, 636280224639, 0, -33579090444], [-6207288287034, 0, 7569267840, 0], [-7264217059696, 0, 3160144623, 0], [3896141278506, 0, -11299810956, 0], [0, 904414069847, 0, 59709310791], [0, 1242749586147, 0, -23036954655], [-2412995306402, 0, 5751855648, 0], [-106883870352, 0, -536097366, 0], [0, 584055730133, 0, -105758149860], [0, 1424207801451, 0, -53900734296], [1639572880804, 0, 11506402818, 0], [0, 1406283540039, 0, 118210764507], [0, -472904412857, 0, 13737984528], [-1099248393618, 0, -10785151242, 0], [0, -1499545107493, 0, 67606481793], [0, -349641408981, 0, -88760050341], [0, -2413008952949, 0, 64091125377], [-8197545214068, 0, 4267247967, 0], [-2028548491184, 0, 21298275594, 0], [1832576801322, 0, -1165751394, 0], [0, 438309279631, 0, -73293953676], [30239177319822, 0, -6226194864, 0], [0, -3440104214037, 0, -80160703491], [0, -2453396603655, 0, 61531654848], [0, -518827403729, 0, -77116045761], [-24836809734264, 0, 13911133929, 0], [0, -339855077781, 0, 250832562804], [0, -961029251349, 0, -81369589341], [-32636549761924, 0, 7553541411, 0], [0, 2216338132039, 0, 136204402428]]
